class TalltyImportExport::ExportPayload
  # header: 导出时候使用的配置模型
  # payload: 对应的单行数据抽取，如果是list属性，则payload是一个array
  attr_accessor :header, :payload, :children, :parent

  def initialize(header:, payload:, parent: nil, children: [])
    @header = header
    @payload = payload
    @children = []
    @parent = parent

    # 递归生成
    if header.is_a?(TalltyImportExport::Attr::ExportHeader)
      @children = header.items.map do |header_item|
        self.class.new(
          header: header_item,
          payload: payload_value(header: header_item, payload: payload),
          parent: self
        )
      end
    else
      if header.children.present?
        @children = header.children.map do |header_item|
          self.class.new(
            header: header_item,
            payload: payload_value(header: header_item, payload: payload),
            parent: self
          )
        end
      end
    end
  end

  def height
    return 0 if payload.blank?

    if header.children.blank?
      # 单属性，如果有值，为1，没有值为0
      payload.present? ? 1 : 0
    else
      # list属性，payload是array
      # 高度是表格里所有属性的height的最大值
      children_height_arr = header.children.map do |header_child|
        Array(payload).map do |payload_child|
          self.class.new(
            header: header_child,
            payload: payload_value(header: header_child, payload: payload_child),
          ).height
        end.sum
      end.max
    end
  end

  # TODO 获取值的方法
  def payload_value header:, payload:
    payload&.dig(header.key)
  end

end
